"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DidKey = void 0;
const ed25519_1 = require("@stablelib/ed25519");
const multiformats_1 = require("multiformats");
const BufferEncoder_1 = require("../../../utils/BufferEncoder");
const MultiBaseEncoder_1 = require("../../../utils/MultiBaseEncoder");
const buffer_1 = require("../../../utils/buffer");
const parse_1 = require("../parse");
const DidDocumentBuilder_1 = require("./DidDocumentBuilder");
const keyTypeResolverMap = {
    ["ed25519" /* ED25519 */]: getEd25519DidDoc,
    ["x25519" /* X25519 */]: getX25519DidDoc,
    ["bls12381g1" /* BLS12381G1 */]: getBls12381g1DidDoc,
    ["bls12381g2" /* BLS12381G2 */]: getBls12381g2DidDoc,
    ["bls12381g1g2" /* BLS12381G1G2 */]: getBls12381g1g2DidDoc,
};
// based on https://github.com/multiformats/multicodec/blob/master/table.csv
const idPrefixMap = {
    234: "bls12381g1" /* BLS12381G1 */,
    235: "bls12381g2" /* BLS12381G2 */,
    236: "x25519" /* X25519 */,
    237: "ed25519" /* ED25519 */,
    238: "bls12381g1g2" /* BLS12381G1G2 */,
};
class DidKey {
    constructor(publicKey, keyType) {
        this.publicKey = buffer_1.Buffer.from(publicKey);
        this.keyType = keyType;
    }
    static fromDid(did) {
        const parsed = parse_1.parseDid(did);
        if (!parsed) {
            throw new Error('Unable to parse did');
        }
        return DidKey.fromFingerprint(parsed.id);
    }
    static fromPublicKey(publicKey, keyType) {
        return new DidKey(buffer_1.Buffer.from(publicKey), keyType);
    }
    static fromPublicKeyBase58(publicKey, keyType) {
        const publicKeyBytes = BufferEncoder_1.BufferEncoder.fromBase58(publicKey);
        return DidKey.fromPublicKey(publicKeyBytes, keyType);
    }
    static fromFingerprint(fingerprint) {
        const { data } = MultiBaseEncoder_1.MultiBaseEncoder.decode(fingerprint);
        const [code, byteLength] = multiformats_1.varint.decode(data);
        const publicKey = buffer_1.Buffer.from(data.slice(byteLength));
        const keyType = idPrefixMap[code];
        if (!keyType) {
            throw new Error(`Unsupported key type from multicodec code '${code}'`);
        }
        return new DidKey(publicKey, keyType);
    }
    get prefixedPublicKey() {
        const codes = Object.keys(idPrefixMap);
        const code = codes.find((key) => idPrefixMap[key] === this.keyType);
        // Create Uint8Array with length of the prefix bytes, then use varint to fill the prefix bytes
        const prefixBytes = multiformats_1.varint.encodeTo(code, new Uint8Array(multiformats_1.varint.encodingLength(code)));
        // Combine prefix with public key
        return buffer_1.Buffer.concat([prefixBytes, this.publicKey]);
    }
    get fingerprint() {
        return `z${BufferEncoder_1.BufferEncoder.toBase58(this.prefixedPublicKey)}`;
    }
    get did() {
        return `did:key:${this.fingerprint}`;
    }
    get didDocument() {
        const resolve = keyTypeResolverMap[this.keyType];
        return resolve(this);
    }
    get publicKeyBase58() {
        return BufferEncoder_1.BufferEncoder.toBase58(this.publicKey);
    }
    get keyId() {
        return `${this.did}#${this.fingerprint}`;
    }
}
exports.DidKey = DidKey;
function getBls12381g2DidDoc(didKey) {
    return getSignatureKeyBase(didKey, {
        id: didKey.keyId,
        type: 'Bls12381G2Key2020',
        controller: didKey.did,
        publicKeyBase58: didKey.publicKeyBase58,
    }).build();
}
function getBls12381g1g2DidDoc(didKey) {
    const g1PublicKey = didKey.publicKey.slice(0, 48);
    const g2PublicKey = didKey.publicKey.slice(48);
    const bls12381g1Key = DidKey.fromPublicKey(g1PublicKey, "bls12381g1" /* BLS12381G1 */);
    const bls12381g2Key = DidKey.fromPublicKey(g2PublicKey, "bls12381g2" /* BLS12381G2 */);
    const bls12381g1KeyId = `${didKey.did}#${bls12381g1Key.fingerprint}`;
    const bls12381g2KeyId = `${didKey.did}#${bls12381g2Key.fingerprint}`;
    const didDocumentBuilder = new DidDocumentBuilder_1.DidDocumentBuilder(didKey.did)
        // BlS12381G1
        .addVerificationMethod({
        id: bls12381g1KeyId,
        type: 'Bls12381G1Key2020',
        controller: didKey.did,
        publicKeyBase58: bls12381g1Key.publicKeyBase58,
    })
        .addAuthentication(bls12381g1KeyId)
        .addAssertionMethod(bls12381g1KeyId)
        .addCapabilityDelegation(bls12381g1KeyId)
        .addCapabilityInvocation(bls12381g1KeyId)
        // BlS12381G2
        .addVerificationMethod({
        id: bls12381g2KeyId,
        type: 'Bls12381G2Key2020',
        controller: didKey.did,
        publicKeyBase58: bls12381g2Key.publicKeyBase58,
    })
        .addAuthentication(bls12381g2KeyId)
        .addAssertionMethod(bls12381g2KeyId)
        .addCapabilityDelegation(bls12381g2KeyId)
        .addCapabilityInvocation(bls12381g2KeyId);
    return didDocumentBuilder.build();
}
function getBls12381g1DidDoc(didKey) {
    return getSignatureKeyBase(didKey, {
        id: didKey.keyId,
        type: 'Bls12381G1Key2020',
        controller: didKey.did,
        publicKeyBase58: didKey.publicKeyBase58,
    }).build();
}
function getX25519DidDoc(didKey) {
    const document = new DidDocumentBuilder_1.DidDocumentBuilder(didKey.did)
        .addKeyAgreement({
        id: didKey.keyId,
        type: 'X25519KeyAgreementKey2019',
        controller: didKey.did,
        publicKeyBase58: didKey.publicKeyBase58,
    })
        .build();
    return document;
}
function getEd25519DidDoc(didKey) {
    const verificationMethod = {
        id: didKey.keyId,
        type: 'Ed25519VerificationKey2018',
        controller: didKey.did,
        publicKeyBase58: didKey.publicKeyBase58,
    };
    const publicKeyX25519 = ed25519_1.convertPublicKeyToX25519(didKey.publicKey);
    const didKeyX25519 = new DidKey(publicKeyX25519, "x25519" /* X25519 */);
    const x25519Id = `${didKey.did}#${didKeyX25519.fingerprint}`;
    const didDocBuilder = getSignatureKeyBase(didKey, verificationMethod);
    didDocBuilder
        .addContext('https://w3id.org/security/suites/ed25519-2018/v1')
        .addContext('https://w3id.org/security/suites/x25519-2019/v1')
        .addVerificationMethod({
        id: `${didKey.did}#${didKeyX25519.fingerprint}`,
        type: 'X25519KeyAgreementKey2019',
        controller: didKey.did,
        publicKeyBase58: didKeyX25519.publicKeyBase58,
    })
        .addKeyAgreement(x25519Id);
    return didDocBuilder.build();
}
function getSignatureKeyBase(didKey, verificationMethod) {
    const keyId = didKey.keyId;
    return new DidDocumentBuilder_1.DidDocumentBuilder(didKey.did)
        .addVerificationMethod(verificationMethod)
        .addAuthentication(keyId)
        .addAssertionMethod(keyId)
        .addCapabilityDelegation(keyId)
        .addCapabilityInvocation(keyId);
}
//# sourceMappingURL=DidKey.js.map